// NODocument.h
//
// NODocument: NODocument class
//			
//
// (c) NiXPS NV, October 2006

#ifndef _NODOCUMENT_H_
#define _NODOCUMENT_H_

/*! \file NODocument.h
    \brief Header file for NODocument class
*/

#include "NTypes.h"
#include "NOXPrefix.h"
#include "NOPartWithResources.h"
#include "NOProgressReporter.h"

// C api
extern "C" {
	DLLIMPORTEXPORT NODocumentHandler NODocument_new(NOHandler* pHandler, NCommon::UInt16 pID);
	DLLIMPORTEXPORT NCommon::UInt32 NODocument_getNumberOfPages(NODocumentHandler* pDocument);
	DLLIMPORTEXPORT NOPageHandler NODocument_getPage(NODocumentHandler* pDocument,  NCommon::UInt32 pNum);
	DLLIMPORTEXPORT NCommon::UInt32 NODocument_getPageNumberOf(NODocumentHandler* pDocument, NOPageHandler pPage);
	DLLIMPORTEXPORT NOPageHandler NODocument_insertPage(NODocumentHandler* pDocument, NOPageHandler pPage,  NCommon::UInt32 pPos/*, void *pReporter*/);
	DLLIMPORTEXPORT NOPageHandler NODocument_createPage(NODocumentHandler* pDocument);
	DLLIMPORTEXPORT NCommon::UInt32 NODocument_searchAndReplaceText(NODocumentHandler* pDocument, NCommon::UTF8Char *pSearch, NCommon::UTF8Char *pReplace, NOFontVectorHandler pMissingFonts/*, void *pReporter*/, NCommon::UInt32 pDryRun);
	DLLIMPORTEXPORT NCommon::UInt32 NODocument_exportToPDF(NODocumentHandler *pDocument, NCommon::UTF8Char *pPDFFileName, NOProgressReporterHandler lProgressReporter);
	DLLIMPORTEXPORT NCommon::UInt32 NODocument_exportToPDFInMemory(NODocumentHandler *pDocument, NCommon::UInt8 *pBuffer, NCommon::UInt32 pBufSize, NCommon::UInt32 *pPDFSize, NOProgressReporterHandler lProgressReporter);
	DLLIMPORTEXPORT NOPackageHandler NODocument_getPackage(NODocumentHandler *pDocument);
	DLLIMPORTEXPORT NCommon::UInt32 NODocument_getDocumentNumber(NODocumentHandler *pDoc);
	DLLIMPORTEXPORT NOXHandler NODocument_getDocumentStructure(NOHandler* pHandler);
	DLLIMPORTEXPORT NCommon::UInt32 NODocument_hasDocumentStructure(NOHandler* pHandler);
	DLLIMPORTEXPORT NCommon::UInt32 NODocument_getFonts(NOHandler* pHandler, NOVectorHandler pFonts);
	DLLIMPORTEXPORT NCommon::UInt32 NODocument_getImages(NOHandler* pHandler, NOVectorHandler pImages);
	DLLIMPORTEXPORT NCommon::UInt32 NODocument_resolveLink(		NOHandler *pHandler,
																NCommon::UTF8Char *pLink,
																NCommon::UInt16 *pPageID,
																NCommon::UInt32 *pObjectID);
};

#ifdef NIXPS_CPP_INTERFACE
// C++ api
namespace NiXPSObjects
{

class NOPage;
class NOThumbnail;
class NOFontVector;
class NOImageVector;
class NOProgressReporter;

/*!
 * \class NODocument NODocument.h "NODocument.h"
 * Class representing a document in an XPS package.
 */
class NODocument: public NOPartWithResources
{
	public:
		// constructor
		NODocument();

		// constructor
		NODocument(NODocumentHandler pHandler);

		// destructor
		virtual ~NODocument();

		/// Returns the number of pages
		NCommon::UInt32 getNumberOfPages();

		/// Returns a given page represented by an NOPage object
		/// @param pNum The location of the page to obtain
		NOPage getPage(const UInt32 pNum);
		
		/// Returns the page number this page has in the document
		/// @param pPage the NOPage object representing the page of which we want to know the page number
		NCommon::UInt32 getPageNumberOf(const NOPage &pPage);
		
		/// Insert a page into a given location
		/// @param pPage the NOPage object representing the page to be inserted
		/// @param pPos the Postion to insert the page at
		NOPage insertPage(const NOPage &pPage, UInt32 pPos/*, NCommon::NProgressReporter *pReporter=0*/);

		/// Creates a new page adding it to the back of the pagelist
		NOPage createPage();

		/// Exports the document to a PDF file
		/// @param pPDFFileName the path of the file to write
		/// @param pProgressReporter This progress reporter object gets polled regularly with % update of the export process
		UInt32 exportToPDF(const UTF8Char *pPDFFileName, NOProgressReporter &pProgressReporter);
		
		/// Exports the document to a PDF, and stores that pdf in memory
		/// @param pBuffer the memory buffer to write to
		/// @param pBufSize size of the memory buffer
		/// @param pPDFSize the full size of the pdf in memory - if the operation fails due to a too small buffer, the full required size is still returned in this variable, it allows to do a second attempt with a bigger buffer
		/// @param pProgressReporter This progress reporter object gets polled regularly with % update of the export process
		UInt32 exportToPDF(UInt8 *pBuffer, const UInt32 pBufSize, UInt32 &pPDFSize, NOProgressReporter &pProgressReporter);

		/// get the entry point to the object model for the document structure
		NOXDocumentStructure getDocumentStructure();

		/// returns wheter the document has structure information
		bool hasDocumentStructure();

		/// Search and replace text in a document
		/// @param pSearch String to search for
		/// @param pReplace String to replace search string with
		/// @param pMissingFonts Font vector that will represent the fonts missing to do the replacement if the search and replace fails
		/// @param pDryRun Do a dry run, i.e. only test the search and replace and don't actually perform the operation
		bool searchAndReplaceText(const UTF8Char *pSearch, const UTF8Char *pReplace, NOFontVector &pMissingFonts, /*NCommon::NProgressReporter *pReporter, */bool pDryRun);
		
		/// Returns the package to which the document belongs
		NOPackage getPackage();
		
		/// Returns the number of this document within its package
		NCommon::UInt32 getDocumentNumber();

		/// Returns the count of the number of fonts in this document and a vector with all the fonts.
		/// @param pFonts The vector to put the fonts in
		UInt32 getFonts(NOFontVector &pFonts);
		
		/// Returns the count of the number of images in this document and a vector with all the images.
		/// @param pImages The vector to put the images in
		UInt32 getImages(NOImageVector &pImages);
		
		/// Returns the page and the object ID the link is pointing to
		UInt32 resolveLink(const UTF8Char *pSearch, UInt16 *pPageID, UInt32 *pObjectID);
};

}
#endif // NIXPS_CPP_INTERFACE

#endif
