// NOPackage.cpp
//
// Object that represents a complete package
//			
// (c) NiXPS NV, October 2006

/*! \file NOPackage.cpp
    \brief Implementation of the NOPackage class
*/

#include "NTypes.h"
#include "NOPartBased.h"
#include "NOPackage.h"
#include "NOCoreProperties.h"
#include "NODocument.h"
#include "NOException.h"
#include "NOString.h"
#include "NOOptions.h"

namespace NiXPSObjects
{

NOPackage::NOPackage(NOPackageHandler pHandler): NOPartWithResources(pHandler)
{
}

NOPackage::~NOPackage()
{
}

/*static*/ NOPackage * NOPackage::createPackage(bool pCreateEmptyDoc, const UTF8Char *pName)
{
	NOPackageHandler lHandler=NOPackage_create(pCreateEmptyDoc, const_cast<UTF8Char *>(pName));

	if (lHandler.mError!=0)
		throw(NOException(lHandler));

	return new NOPackage(lHandler);
}

/*static*/ NOPackage * NOPackage::createFromImage(const UTF8Char *pFilename)
{
	NOPackageHandler lHandler = NOPackage_createFromImage(const_cast<UTF8Char *>(pFilename));

	if (lHandler.mError!=0)
		throw(NOException(lHandler));

	return new NOPackage(lHandler);
}

/*static*/ void NOPackage::destroyPackage(NOPackage *pXPSPackage)
{	
	NOPackage_destroyPackage(&pXPSPackage->mHandler);
	delete pXPSPackage;
}

/*static*/ NOPackage * NOPackage::readPackageFromFile(const UTF8Char *pFilename)
{
	NOPackageHandler lHandler=NOPackage_readPackageFromFile(const_cast<UTF8Char *>(pFilename));
	
	if (lHandler.mError!=0)
		throw(NOException(lHandler));

	return new NOPackage(lHandler);
}

/*static*/ NOPackage * NOPackage::readPackageFromBuffer(const UTF8Char *pFilename, const UInt8* pBuffer, const UInt32 pSize)
{
	NOPackageHandler lHandler=NOPackage_readPackageFromBuffer(const_cast<UTF8Char *>(pFilename), (UInt8 *) pBuffer, pSize);
	
	if (lHandler.mError!=0)
		throw(NOException(lHandler));

	return new NOPackage(lHandler);
}

void NOPackage::writePackageToFile(const UTF8Char *pFilename) // const
{
	NOPackage_writePackageToFile(&mHandler,const_cast<UTF8Char *>(pFilename));
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

UInt32 NOPackage::writePackageToBuffer(UInt8 *pBuffer, const UInt32 pBufSize, UInt32 &pFileSize)
{
	UInt32 lRet = NOPackage_writePackageToBuffer(&mHandler, pBuffer, pBufSize, &pFileSize);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}
	
void NOPackage::writePackageToFile() 
{
	NOPackage_save(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

// returns the filename
bool NOPackage::hasOutputFilename() const
{
	UInt32 lRet=NOPackage_hasFilename((NOPackageHandler *) &mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

// returns the filename
const UTF8String NOPackage::getOutputFilename() const
{
	NOString lString;
	NOPackage_getFilename((NOPackageHandler *) &mHandler,&lString.mHandler);
	
	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lString.getUTF8String();
}

// set the filename
void NOPackage::setOutputFilename(const UTF8Char *pFilename)
{
	NOPackage_setFilename(&mHandler,const_cast<UTF8Char *>(pFilename));

	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

// returns if the package is 'dirty' - meaning that new stuff sits in memory 
bool NOPackage::getDirty()
{
	UInt32 lRet=NOPackage_getDirty(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

void NOPackage::setDirty(bool pDirty)
{
	NOPackage_setDirty(&mHandler,pDirty);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

// returns true if we have a NOCoreProperties object
bool NOPackage::hasCoreProperties()
{
	UInt32 lRet=NOPackage_hasCoreProperties(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

// returns a NOCoreProperties object
NOCoreProperties NOPackage::getCoreProperties()
{
	NOCorePropertiesHandler lHandler=NOPackage_getCoreProperties(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return NOCoreProperties(lHandler);
}

NCommon::UInt32 NOPackage::getNumberOfDocuments()
{
	UInt32 lRet=NOPackage_getNumberOfDocuments(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

// returns a NOCoreProperties object
NODocument NOPackage::getDocument(const UInt32 pNum)
{	
	NODocumentHandler lHandler=NOPackage_getDocument(&mHandler, pNum);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return NODocument(lHandler);
}

// adds a document
void NOPackage::addDocument(const NODocument &pDoc, NOProgressReporter &pReporter)
{	
	NOPackage_addDocument(&mHandler, (NODocumentHandler) pDoc.mHandler, pReporter.mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

// returns the document number this doc has in the xps package
NCommon::UInt32 NOPackage::getDocumentNumberOf(const NODocument &pDoc)
{
	UInt32 lRet=NOPackage_getDocumentNumberOf(&mHandler, (NODocumentHandler) pDoc.mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

bool NOPackage::hasThumbnail()
{
	UInt32 lRet=NOPackage_hasThumbnail(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

// returns the NOThumbnail object
NOThumbnail NOPackage::getThumbnail()
{
	NOThumbnailHandler lHandler=NOPackage_getThumbnail(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return NOThumbnail(lHandler);
}

NCommon::UInt32 NOPackage::getNumberOfPages()
{
	UInt32 lRet=NOPackage_getNumberOfPages(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

NCommon::UInt32 NOPackage::getFonts(NOFontVector &pFonts)
{
	UInt32 lRet = NOPackage_getFonts(&mHandler,pFonts.mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

NCommon::UInt32 NOPackage::getImages(NOImageVector &pImages)
{
	UInt32 lRet = NOPackage_getImages(&mHandler,pImages.mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

NCommon::UInt32 NOPackage::getParts(NOVector &pParts)
{
	UInt32 lRet = NOPackage_getParts(&mHandler,pParts.mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

UInt32 NOPackage::exportToPDF(const UTF8Char *pPDFFileName, NOProgressReporter &pProgressReporter)
{
	UInt32 lRet = NOPackage_exportToPDF(&mHandler,const_cast<UTF8Char *>(pPDFFileName),pProgressReporter.mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

UInt32 NOPackage::exportToPDFWithOptions(const UTF8Char *pPDFFileName, NOProgressReporter &pProgressReporter, NOOptions &pOptions)
{
	UInt32 lRet = NOPackage_exportToPDFWithOptions(&mHandler,const_cast<UTF8Char *>(pPDFFileName),pProgressReporter.mHandler,pOptions.mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}
	
UInt32 NOPackage::exportToPDFWithOptions(UInt8 *pBuffer, const UInt32 pBufSize, UInt32 &pPDFSize, NOProgressReporter &pProgressReporter, NOOptions &pOptions)
{
	UInt32 lRet = NOPackage_exportToPDFWithOptionsInMemory(&mHandler, pBuffer, pBufSize, &pPDFSize, pProgressReporter.mHandler,pOptions.mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}


}
